#include <stdio.h>

typedef struct node {
  int data;
  struct node *left, *right;
} *Tree;

Tree readTreeFromString(char *line,int *pos)
{
  char ch;

  while(isspace(line[*pos]))
    (*pos)++;
  if(isdigit(line[*pos]))
    {
      Tree t = (Tree)malloc(sizeof(struct node));

      t->data = line[(*pos)++]-'0';
      while(isdigit(line[*pos]))
	t->data = 10 * t-> data + line[(*pos)++]-'0';
      t->left = t->right = NULL;
      return t;
    }
  else if(line[*pos]=='(') 
    {
      Tree t = (Tree)malloc(sizeof(struct node));
      (*pos)++;
      if((t->left = readTreeFromString(line,pos))==NULL)
	 return NULL;
      if(line[*pos]!=',') {
	fprintf(stderr,"Error: Ill-formed tree. Expected ','\n");
	return NULL;
      }
      (*pos)++;
      if((t->right = readTreeFromString(line,pos))==NULL)
	return NULL;
      if(line[*pos]!=')') {
	fprintf(stderr,"Error: Ill-formed tree. Expected ')'\n");
	return NULL;
      }
      (*pos)++;
      return t;
    }
  else {
    fprintf(stderr,"Error: Ill-formed tree. Expected '(' or digit.\n");
    return NULL;
  }
}

Tree readTree(FILE *f)
{
  char line[257];
  int pos = 0;

  if(fgets(line,257,f)==NULL)
    return NULL;
  return readTreeFromString(line,&pos);
}

void freeTree(Tree t)
{
  if(t) {
    freeTree(t->left);
    freeTree(t->right);
    free(t);
  }
}

void printTree(Tree t,FILE *f)
{
  if(t)
    if(t->left) {
      putc('(',f);
      printTree(t->left,f);
      putc(',',f);
      printTree(t->right,f);
      putc(')',f);
    }
    else 
      fprintf(f,"%d",t->data);
}

int treeEqual(Tree t1, Tree t2)
{
  if(!t1)
    return !t2;
  if(!t2)
    return 0;
  if(t1->left)
    return (t2->left && treeEqual(t1->left,t2->left) &&
	    treeEqual(t1->right,t2->right));
  return (!t2->left && t1->data == t2->data);
}

int removeSubtree(Tree *t, Tree d)
{
  if(!*t)
    return 0;
  if(treeEqual((*t)->left,d)) {
    Tree temp = *t;
    *t = (*t)->right;
    freeTree(temp->left);
    free(temp);
    return 1;
  }
  if(treeEqual((*t)->right,d)) {
    Tree temp = *t;
    *t = (*t)->left;
    freeTree(temp->right);
    free(temp);
    return 1;
  }
  if(removeSubtree(&((*t)->left),d))
    return 1;
  return removeSubtree(&((*t)->right),d);
}


int main(int argc,char **argv)
{
  Tree t,d;
  int pos = 0;
  
  if(argc!=2) {
    fprintf(stderr,"Usage: %s <tree to remove>.\n", argv[0]);
    exit(1);
  }
  if((d=readTreeFromString(argv[1],&pos))==NULL) {
    fprintf(stderr,"Error: Could not read tree to remove.\n");
    exit(1);
  }
  while((t = readTree(stdin))!=NULL) 
    if(treeEqual(t,d))
      printf("Warning: cannot delete entire tree.\n");
    else {
      removeSubtree(&t,d);
      printTree(t,stdout);
      putchar('\n');
      freeTree(t);
    }
  freeTree(d);
  return 0;
}
